/*
    $Id: PrefWindowController.java,v 1.4 2003/08/26 16:50:55 admin Exp $
    -----
	
    SSTP Bottle Client for MacOS X, using protocol SLPP2.0
    Copyright (C) 2001-2002 phonohawk.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package gui.pref;
import gui.pref.*;
import core.*;
import action.*;
import util.*;
import com.apple.cocoa.foundation.*;
import com.apple.cocoa.application.*;
import java.io.*;
import java.util.*;
import java.lang.reflect.*;

public class PrefWindowController extends NSWindowController {
    private static PrefWindowController _sharedInstance = null;

    NSTabView tabview;

    // General
    NSButton check_playsound;
    NSPopUpButton listbutton_sounds;
    NSButton button_playsound;

    NSTextField iniscript_script;
    
    NSTableView ghostspec_table;
    PrefGhostSpecTableDataSource ghostspec_table_datasource; // Not an IBOutlet!
    NSButton ghostspec_remove;

    NSTextField bottle_length_limit;

    // Server
    NSTextField botserver_server;
    NSTextField botserver_slppport;
    NSTextField botserver_httpport;
    
    NSTextField sstpserver_server;
    NSTextField sstpserver_sstpport;

    NSButton proxy_use_proxy;
    NSTextField proxy_http_proxy_host;
    NSTextField proxy_http_proxy_port;

    // Action
    NSWindow action_addaction_window;
    NSTableView action_installed;
    NSButton action_remove;
    NSButton action_configure;
    NSButton action_move_upper;
    NSButton action_move_downer;

    // AddAction
    NSPopUpButton addaction_list;
    NSTextField addaction_description;

    // SSTP Error
    NSMatrix sstperror_503_select;
    NSTextField sstperror_503_substitution;

    // Log
    NSButton log_http_enabled;
    NSTextField log_http_port;
    NSButton log_http_basicauth_enabled;
    NSForm log_http_basicauth_form;
    
    public static PrefWindowController sharedPrefWindowController() {
	if (_sharedInstance == null) {
	    _sharedInstance = new PrefWindowController();
	}
	return _sharedInstance;
    }
    
    public PrefWindowController() {
	super("Pref");
	loadWindow();
	NSUserDefaults defaults = NSUserDefaults.standardUserDefaults();

	// ^u̕d
	String tab_id = (String)defaults.objectForKey("pref.tab.selected");
	if (tab_id != null) {
	    tabview.selectTabViewItemWithIdentifier(tab_id);
	}

	/* General  */
	
	// CXg[ĂTEht@CăXg\
	listbutton_sounds.removeAllItems();
	NSArray libraries = NSPathUtilities.searchPathForDirectoriesInDomains(
	    NSPathUtilities.LibraryDirectory,
	    NSPathUtilities.LocalDomainMask + NSPathUtilities.SystemDomainMask + NSPathUtilities.UserDomainMask,
	    true);
	int n_libraries = libraries.count();
	for (int i = 0;i < n_libraries;i++) {
	    File sounddir = new File((String)libraries.objectAtIndex(i),"Sounds");
	    if (!sounddir.exists() || !sounddir.isDirectory()) continue;
	    
	    File[] files = sounddir.listFiles();
	    for (int j = 0;j < files.length;j++) {
		// gq蕥̂TEhB
		String soundname = NSPathUtilities.stringByDeletingPathExtension(files[j].getName());
		listbutton_sounds.addItem(soundname);
	    }
	}
	
	// ݒ肳ĂTEh\
	String soundname = (String)defaults.objectForKey("sound_bottle_receive name");
	if (soundname != null) listbutton_sounds.selectItemWithTitle(soundname);
		
	// enable/disable
	check_playsound.setIntValue(defaults.integerForKey("sound_bottle_receive enabled"));
	check_playsound(check_playsound); // ԍXV
	
	// initial script
	String initialscript = (String)defaults.objectForKey("iniscript.script");
	iniscript_script.setStringValue(initialscript == null ? "\u00a5t\u00a5u\u00a5s[10]\u00a5h\u00a5s[0]|\u00a5e" : initialscript);
	
	// ghost spec
	ghostspec_table_datasource = new PrefGhostSpecTableDataSource();
	NSArray ghostspec_array = defaults.arrayForKey("ghostspec.table");
	if (ghostspec_array == null) {
	    ghostspec_table_datasource.setArray(createDefaultGhostNamesArray());
	}
	else {
	    ghostspec_table_datasource.setArray(newVectorFromNSArray(ghostspec_array));
	}
        ghostspec_table.setDataSource(ghostspec_table_datasource);
	ghostspec_table.setDelegate(this);
	updateGhostSpecControlsState();

	// Bottle Length Limit
	int bottle_len = defaults.integerForKey("bottle-length-limit.limit");
	bottle_length_limit.setIntValue(
	    defaults.objectForKey("bottle-length-limit.limit") == null ? 1900 : bottle_len);

	
	/* Server */
	
	// bottle server
	String botservername = (String)defaults.objectForKey("bottleserver.servername");
	botserver_server.setStringValue(botservername == null ? "bottle.mikage.to" : botservername);
	int slppport = defaults.integerForKey("bottleserver.slppport");
	botserver_slppport.setIntValue(slppport == 0 ? 9871 : slppport);
	int httpport = defaults.integerForKey("bottleserver.httpport");
	botserver_httpport.setIntValue(httpport == 0 ? 80 : httpport);
	
	// sstp server
	String sstpservername = (String)defaults.objectForKey("sstpserver.servername");
	sstpserver_server.setStringValue(sstpservername == null ? "localhost" : sstpservername);
	int sstpport = defaults.integerForKey("sstpserver.sstpport");
	sstpserver_sstpport.setIntValue(sstpport == 0 ? 11000 : sstpport);

	// proxy
	proxy_use_proxy.setIntValue(defaults.integerForKey("proxy.http.use"));
	String http_proxy_host = (String)defaults.objectForKey("proxy.http.host");
	proxy_http_proxy_host.setStringValue(http_proxy_host == null ? "" : http_proxy_host);
	proxy_http_proxy_port.setIntValue(defaults.integerForKey("proxy.http.port"));

	
	/* Action */

	// installed actions
	action_installed.setDataSource(InstalledActionsTableDataSource.sharedDataSource());
	action_installed.setDelegate(this);
	action_installed.setDoubleAction(
	    new NSSelector("configure_action", new Class[] {NSObject.class}));
	action_installed.setTarget(this);
	update_action_controls_state();


	/* SSTP Error */

	// 503 Service Unavailable
	sstperror_503_select.selectCellWithTag( // ftHg0"Give up"
	    defaults.integerForKey("sstp-error.503.select"));
	String sstperror_503_ghostname =
	    (String)defaults.objectForKey("sstp-error.503.substitution");
	sstperror_503_substitution.setStringValue(
	    sstperror_503_ghostname == null ? "-UNINSTALLED-" : sstperror_503_ghostname);


	/* Log */

	// http-based log browser
	log_http_enabled.setIntValue(
	    defaults.integerForKey("http-log-browser.enabled"));
	log_http_port.setIntValue(
	    defaults.objectForKey("http-log-browser.port") == null ?
	    19010 : defaults.integerForKey("http-log-browser.port"));
	log_http_basicauth_enabled.setIntValue(
	    defaults.integerForKey("http-log-browser.basic-auth.enabled"));
	if (defaults.objectForKey("http-log-browser.basic-auth.id") == null) {
	    log_http_basicauth_form.cellAtIndex(0).setStringValue("botcli");
	    log_http_basicauth_form.cellAtIndex(1).setStringValue("botcli");
	}
	else {
	    log_http_basicauth_form.cellAtIndex(0).setStringValue(
		(String)defaults.objectForKey("http-log-browser.basic-auth.id"));
	    log_http_basicauth_form.cellAtIndex(1).setStringValue(
		(String)defaults.objectForKey("http-log-browser.basic-auth.password"));
	}
    }
	
    public static NSArray newNSArrayFromVector(Vector vec) {
	final int size_vec = vec.size();
	NSMutableArray array = new NSMutableArray(); // TCY̐ݒ肪oȂI
	for (int i = 0; i < size_vec; i++) {
	    array.addObject(vec.elementAt(i));
	}
	return array;
    }
    
    public static Vector newVectorFromNSArray(NSArray array) {
	final int size_array = array.count();
	Vector vec = new Vector(size_array);
	for (int i = 0; i < size_array; i++) {
	    vec.add(array.objectAtIndex(i));
	}
	return vec;
    }

    public static Vector newVectorFromNSEnumerator(NSEnumerator enu) {
	Vector result = new Vector();
	for (; enu.hasMoreElements(); ) {
	    result.add(enu.nextElement());
	}
	return result;
    }
    
    public static Vector createDefaultGhostNamesArray()
    {
		Vector array = new Vector();
		array.add("\u3055\u304f\u3089"); // 
		array.add("\u53cc\u8449"); // ot
		array.add("\u305b\u308a\u3053"); // 肱
		array.add("\u7406\u5922"); // 
		array.add("\u307e\u3086\u3089"); // ܂
		array.add("\u306d\u3053\u3053"); // ˂
		array.add("\u3064\u3050\u306a"); // 
		array.add("\u82b1\u3061\u3083\u3093"); // Ԃ
		array.add("\u5b89\u5b50\u3055\u3093"); // q
		array.add("\u7f8e\u8033"); // 
		array.add("\u307f\u304b\u3093"); // ݂
		array.add("\u3088\u308f\u306d"); // 
		return array;
    }
    
    public void updateGhostSpecControlsState() {
	boolean selectedOne = (ghostspec_table.selectedRow() != -1);
	ghostspec_remove.setEnabled(selectedOne);
    }

    public void update_action_controls_state() {
	// removéAłĂ鍀ڂΎgp\B
	// configuréAĂꍇɂ̂ݎgp\B
	// (_Iɕs\Ɖ]桂ł͂ȂB̍ڂ܂
	// configure{^Ă܂ӂƁcB)
	int n_selected = action_installed.numberOfSelectedRows();
	action_remove.setEnabled(n_selected > 0);
	action_configure.setEnabled(n_selected == 1);

	// move uppeŕAȏ̍ڂĂāA
	// Ă钆ōł̍ڂ擪łȂΎgp\B
	// move downerupperƓł邪AĂ钆
	// ł̍ڂłȂΎgp\B
	if (n_selected == 0) {
	    action_move_upper.setEnabled(false);
	    action_move_downer.setEnabled(false);
	}
	else {
	    Vector selected_rows = newVectorFromNSEnumerator(
		action_installed.selectedRowEnumerator());

	    int upmost_selected =
		((Integer)selected_rows.get(0)).intValue();
	    int downmost_selected =
		((Integer)selected_rows.get(selected_rows.size()-1)).intValue();

	    action_move_upper.setEnabled(upmost_selected > 0);
	    action_move_downer.setEnabled(
		downmost_selected < action_installed.numberOfRows() - 1);
	}
    }
    
    /******** delegate methods ********/
    public boolean windowShouldClose(Object sender)
    {
	NSUserDefaults defaults = NSUserDefaults.standardUserDefaults();
        // defaultsɕۑ܂B

	// ^u
	defaults.setObjectForKey(
	    tabview.selectedTabViewItem().identifier(),
	    "pref.tab.selected");
	
	// sound
	String selectedTitle = listbutton_sounds.titleOfSelectedItem();
	defaults.setObjectForKey(selectedTitle,"sound_bottle_receive name");
	defaults.setIntegerForKey(check_playsound.intValue(),"sound_bottle_receive enabled");
		
	// initial script
	defaults.setObjectForKey(iniscript_script.stringValue(),"iniscript.script");
		
	// ghost spec
	defaults.setObjectForKey(newNSArrayFromVector(ghostspec_table_datasource.getArray()),"ghostspec.table");
	NSNotificationCenter.defaultCenter().postNotification("BCGhostSpecModified",this);

	// bottle length limit
	defaults.setIntegerForKey(bottle_length_limit.intValue(), "bottle-length-limit.limit");
	NSNotificationCenter.defaultCenter().postNotification("BCBottleLengthLimitChanged", this);

	/* Server */
	
	// bottle server
	defaults.setObjectForKey(botserver_server.stringValue(),"bottleserver.servername");
	defaults.setIntegerForKey(botserver_slppport.intValue(),"bottleserver.slppport");
	defaults.setIntegerForKey(botserver_httpport.intValue(),"bottleserver.httpport");
		
	// sstp server
	defaults.setObjectForKey(sstpserver_server.stringValue(),"sstpserver.servername");
	defaults.setIntegerForKey(sstpserver_sstpport.intValue(),"sstpserver.sstpport");

	// proxy
	defaults.setIntegerForKey(proxy_use_proxy.intValue(),"proxy.http.use");
	defaults.setObjectForKey(proxy_http_proxy_host.stringValue(),"proxy.http.host");
	defaults.setIntegerForKey(proxy_http_proxy_port.intValue(),"proxy.http.port");

	
	/* SSTP Error */

	// 503 Service Unavailable
	defaults.setIntegerForKey(
	    sstperror_503_select.selectedCell().tag(), "sstp-error.503.select");
	defaults.setObjectForKey(
	    sstperror_503_substitution.stringValue(), "sstp-error.503.substitution");

	/* Log */

	// http-based log browser
	defaults.setIntegerForKey(log_http_enabled.intValue(), "http-log-browser.enabled");
	defaults.setIntegerForKey(log_http_port.intValue(), "http-log-browser.port");
	defaults.setIntegerForKey(log_http_basicauth_enabled.intValue(), "http-log-browser.basic-auth.enabled");
	defaults.setObjectForKey(log_http_basicauth_form.cellAtIndex(0).stringValue(), "http-log-browser.basic-auth.id");
	defaults.setObjectForKey(log_http_basicauth_form.cellAtIndex(1).stringValue(), "http-log-browser.basic-auth.password");
	if (log_http_enabled.intValue() != 0) {
	    Foundation.sharedFoundation().startHttpd();
	}
	else {
	    Foundation.sharedFoundation().stopHttpd();
	}
		
	defaults.synchronize();
	
	ActionManager.sharedManager().saveActions();
	return true; // allow closing
    }
    
    public void tableViewSelectionDidChange(NSNotification aNotification) {
	NSTableView the_table = (NSTableView)aNotification.object();
	if (the_table == ghostspec_table) {
	    updateGhostSpecControlsState();
	}
	else if (the_table == action_installed) {
	    update_action_controls_state();
	}
    }
    
    /********* action methods *********/
    public void check_playsound(Object sender)
    {
		boolean enabled = (check_playsound.intValue() == 1);
		listbutton_sounds.setEnabled(enabled);
		button_playsound.setEnabled(enabled);
		Foundation.sharedFoundation().setEnabledOfSoundOfReceiveBottle(enabled);
    }

    public void listbutton_sounds(Object sender)
    {
		Foundation foundation = Foundation.sharedFoundation();
		
		foundation.setSoundOfReceivingBottle(listbutton_sounds.titleOfSelectedItem());
		foundation.playSoundOfReceiveBottle();
    }
    
    public void playsound(Object sender)
    {
		Foundation.sharedFoundation().playSoundOfReceiveBottle();
    }
    
    public void botserver_reset(Object sender)
    {
		botserver_server.setStringValue("bottle.mikage.to");
		botserver_slppport.setIntValue(9871);
		botserver_httpport.setIntValue(80);
    }
    
    public void sstpserver_reset(Object sender)
    {
		sstpserver_server.setStringValue("localhost");
		sstpserver_sstpport.setIntValue(11000);
    }
    
    public void iniscript_reset(Object sender)
    {
		iniscript_script.setStringValue("\u00a5t\u00a5u\u00a5s[10]\u00a5h\u00a5s[0]|\u00a5e");
    }
    
    public void ghostspec_reset(Object sender)
    {
		ghostspec_table_datasource.setArray(createDefaultGhostNamesArray());
		ghostspec_table.reloadData();
    }
    
    public void ghostspec_add(Object sender)
    {
		ghostspec_table_datasource.addItem();
		ghostspec_table.reloadData();
	
		// Ō̍ڂIҏWԂɂ
		int last_row_index = ghostspec_table.numberOfRows()-1;
		ghostspec_table.selectRow(last_row_index,false); // ̂ȂƃG[ɂȂB
		ghostspec_table.editLocation(0,last_row_index,null,true);
    }
    
    public void ghostspec_remove(Object sender)
    {
		ghostspec_table_datasource.removeItemAtIndex(ghostspec_table.selectedRow());
		ghostspec_table.reloadData();
    }

    public void add_action(Object sender) {
	// AddActionEChEV[gƂĊJB
	
	// ANVNX̃Xg\z
	addaction_list.removeAllItems();
	for (Iterator ite = ActionManager.getActionNameList().iterator();
	     ite.hasNext(); ) {
	    addaction_list.addItem((String)ite.next());
	}
	addaction_selected(this);
	
	NSApplication.sharedApplication().beginSheet(
	    action_addaction_window,window(),this,null,null);
    }

    public void addaction_selected(Object sender) {
	// ݐĂANV͂ǂꂩH
	String name = addaction_list.titleOfSelectedItem();

	// ̐\
	addaction_description.setStringValue(ActionManager.getShortDescription(name));
    }

    public void addaction_cancel(Object sender) {
	NSApplication.sharedApplication().endSheet(
	    action_addaction_window);
	action_addaction_window.close();
    }

    public void addaction_ok(Object sender) {
	NSApplication.sharedApplication().endSheet(
	    action_addaction_window);
	
	// ꂽANV͂ǂꂩH
	String name = addaction_list.titleOfSelectedItem();
	action_addaction_window.close();

	// ̖RXgN^ŃIuWFNg𐶐AconfigureĂԁB
	try {
	    BottleAction action = (BottleAction)ActionManager.getActionClass(name).newInstance();
	    Method callback_add_action = getClass().getMethod(
		"callback_add_action",new Class[] {BottleAction.class});
	    
	    action.configure(this,callback_add_action,null);
	} catch (Exception e) {}
    }

    public void callback_add_action(BottleAction action) {
	// ĂANV邩H
	if (action_installed.selectedRow() == -1) {
	    // B
	    ActionManager.sharedManager().installAction(action);
	}
	else {
	    // LBŌɐꂽڂ̎ցB(̈ʒuɂ͉ǂPnLB)
	    ActionManager.sharedManager().installAction(
		action,action_installed.selectedRow()+1);
	}
	action_installed.reloadData();
	ActionManager.sharedManager().saveActions();
    }

    public void configure_action(Object sender) {
	try {
	    BottleAction action = ActionManager.
		sharedManager().getNthAction(action_installed.selectedRow());
	    Method callback = getClass().getMethod(
		"callback_action_reconfigured",new Class[] {BottleAction.class});
	    
	    action.configure(this,callback,null);
	} catch (Exception e) {}
    }

    public void callback_action_reconfigured(BottleAction action) {
	action_installed.reloadData();
	ActionManager.sharedManager().saveActions();
    }

    public void remove_action(Object sender) {
	// ĂSẴANV폜B
	ActionManager.sharedManager().removeActions(
	    new NSEnumeratingIterator(action_installed.selectedRowEnumerator()));
	action_installed.reloadData();
	ActionManager.sharedManager().saveActions();
    }

    public void move_action_upper(Object sender) {
	// ĂSẴANVÂɈړB
	ActionManager.sharedManager().moveUpper(
	    new NSEnumeratingIterator(action_installed.selectedRowEnumerator()));

	// ʒuÂɈړB
	Vector old_selected = newVectorFromNSEnumerator(
	    action_installed.selectedRowEnumerator());
	action_installed.deselectAll(this);
	for (Iterator ite = old_selected.iterator(); ite.hasNext(); ) {
	    int idx = ((Integer)ite.next()).intValue();
	    action_installed.selectRow(idx - 1, true);
	}
	
	action_installed.reloadData();
    }

    public void move_action_downer(Object sender) {
	// ĂSẴANVÂɈړB
	ActionManager.sharedManager().moveDowner(
	    new NSEnumeratingIterator(action_installed.selectedRowEnumerator()));

	// ʒuÂɈړB
	Vector old_selected = newVectorFromNSEnumerator(
	    action_installed.selectedRowEnumerator());
	action_installed.deselectAll(this);
	for (Iterator ite = old_selected.iterator(); ite.hasNext(); ) {
	    int idx = ((Integer)ite.next()).intValue();
	    action_installed.selectRow(idx + 1, true);
	}
	
	action_installed.reloadData();
    }
}
